//  +build fasthttp

package jwtmdl

import (
	"fmt"
	"strings"

	"github.com/valyala/fasthttp"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	jwt "github.com/dgrijalva/jwt-go"
)

// DecodeTokenWithJWTKey decode token
func DecodeTokenWithJWTKey(req *fasthttp.Request, jwtKey string) (jwt.MapClaims, error) {
	tokenFromRequest := string(req.Header.Peek("Authorization"))
	tokenArray := strings.Split(tokenFromRequest, "Bearer")
	if len(tokenArray) <= 1 {
		return nil, errormdl.Wrap("Provided JWT token is nil or invalid ")
	}
	tokenFromRequest = strings.Trim(tokenArray[1], " ")
	// get data i.e.Claims from token
	token, err := jwt.Parse(tokenFromRequest, func(token *jwt.Token) (interface{}, error) {
		// Don't forget to validate the alg is what you expect:
		_, ok := token.Method.(*jwt.SigningMethodHMAC)
		if !ok {
			return nil, fmt.Errorf("Unexpected signing method: %v", token.Header["alg"])
		}
		return []byte(jwtKey), nil
	})

	if err != nil {
		loggermdl.LogError("Error while parsing JWT Token: ", err)
		return nil, err
	}

	claims, ok := token.Claims.(jwt.MapClaims)
	if !errormdl.CheckBool1(ok) {
		loggermdl.LogError("Error while parsing claims to MapClaims")
		return nil, errormdl.Wrap("Error while getting claims")
	}

	return claims, nil
}

// DecodeToken decode token
func DecodeToken(req *fasthttp.Request) (jwt.MapClaims, error) {
	tokenFromRequest := string(req.Header.Peek("Authorization"))
	loggermdl.LogError(tokenFromRequest)
	tokenArray := strings.Split(tokenFromRequest, "Bearer")
	if len(tokenArray) <= 1 {
		return nil, errormdl.Wrap("Provided JWT token is nil or invalid ")
	}
	tokenFromRequest = strings.Trim(tokenArray[1], " ")
	// get data i.e.Claims from token
	token, err := jwt.Parse(tokenFromRequest, func(token *jwt.Token) (interface{}, error) {
		// Don't forget to validate the alg is what you expect:
		_, ok := token.Method.(*jwt.SigningMethodHMAC)
		if !ok {
			return nil, fmt.Errorf("Unexpected signing method: %v", token.Header["alg"])
		}
		return []byte(GlobalJWTKey), nil
	})

	if err != nil {
		loggermdl.LogError("Error while parsing JWT Token: ", err)
		return nil, err
	}

	claims, ok := token.Claims.(jwt.MapClaims)
	if !errormdl.CheckBool1(ok) {
		loggermdl.LogError("Error while parsing claims to MapClaims")
		return nil, errormdl.Wrap("Error while getting claims")
	}
	return claims, nil
}
